#!/usr/bin/env  perl 

##########################################################################
#
# Licensed Materials - Property of IBM
# 5724-T10
#
# (C) Copyright IBM Corp. 2003,2010. All Rights Reserved.
#
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
##########################################################################

use Data::Dumper;
use File::Basename;

use Getopt::Std;

getopt('fbo');


my %AFN;
my  %SC;
my %AFH;
my %outH;
my %rel;

$ascii_file=$opt_f;
$bdf_file = $opt_b;
$outFileName=$opt_o;

$temp_bdf_file="/tmp/" . basename($bdf_file) . ".upd";


#&usage if ($opt_f == '' || $opt_b == '' || $opt_o == '' );
&usage if ($opt_f eq  '' || $opt_b eq '' || $opt_o eq '' );

sub  trim_bdf
{
   `cat $bdf_file | nawk 'BEGIN {FS =":"}{print \$1,\$2, \$3}' |nawk 'BEGIN {FS = " "}{ print \$1,\$2, \$3, \$4}' | sort|sed 's/))\$//g' |uniq > $temp_bdf_file`;
    if ($? != 0 )
    {
       print "Unable to create temporary file $temp_bdf_file. Please check disk space or permissions \n";
       exit 1;
    }
}



sub usage {
    print("USAGE : generate_cbrs_cfg.pl  -f <ascii_base_file> -b <bdf file> -o <output cbrs cfg file> \n ");
    print "generate_cbrs_cfg.pl -f 99.99.99.99.base -b pp_sta_20100803T000000_NODE01_001.bdf  -o MTX01.NODE01.MTX01B2CBRS1.cbrs.cfg \n";

    exit _RET_CODE;
}

sub open_ascii_view
{

unless (open(ASCII_FILE, "$ascii_file"))
{
   die("unable to open $ascii_file \n");
}

}

sub open_bdf_upd_file
{
unless (open(BDF_FILE, "${temp_bdf_file}")) 
{
   die("unable to open ${temp_bdf_file} \n");
}
}



sub load_ascii_view
{
   while (<ASCII_FILE>)
   {
#     if ($_ =~ m/set[ \/0-9A-Za-z,]*applicationFramerName (\w+)(\/+)(\d+)/  )
     #There are set Imh/6 applicationFramerName Bts/59 BtsIf/14,projectedNumberOfLinks lines
     # and that will make the value for this hash as Imh/6. That is not intended. Hence added txPathTrace.

     if ($_ =~ m/set (\w+)(\/)(\w+)[ \/0-9A-Za-z,]*applicationFramerName (\w+)(\/)(\w+),[txPathTrace|timeSlots]/  )
     {
#        print "found Framer name $1, $2 , $3 ,$4, $5, $6 \n";
       
        $AFN{"$4$5$6"} = "$1$2$3";

     }
	
     if ($_ =~ m/set (\w+)(\/)(\w+)[ \/0-9A-Za-z,]*mainCard(\s+)(\w+)(\s+)(\w+)(\/)(\d+)/  )
     {
#        print "found maincard name $1, $2 , $3 ,$5, $7, $9 \n";
        $AFH{"$1$2$3"}->{mainCard} = "$5 $7 $9";
      }


     if ($_ =~ m/set (\w+)(\/)(\w+)[ \/0-9A-Za-z,]*interfaceName(\s+)([!]?)(\w+)(\/)(\d+)/  ) 
     {
          #$5 will be "!" in one case and null in others
          if ($1 ne  "Sbs" )
          {
            $AFH{"$1$2$3"}->{interfaceName} = "$6$7$8";
#        print "found nonSbs interface name name $1, $2 , $3 ,$5, $6, $7, $8 \n";
           }
      }
     if ($_ =~ m/set (\w+)(\/)(\w+)(,\d+)[ \/0-9A-Za-z,]*interfaceName(\s+)([!]?)(\w+)(\/)(\d+)/  ) 
     {
        #print "found interface name name $1, $2 , $3 ,$5, $6, $7, $8 \n";
          #$5 will be "!" in one case and null in others
        $AFH{"$1$2$3$4"}->{interfaceName} = "$7$8$9";
    }

     if ($_ =~ m/set (\w+)(\/)(\w+)[ \/0-9A-Za-z,]*mainLp(\s+)(\w+)(\/)(\d+)/  )
     {
#        print "found interface mainLp $1, $2 , $3 ,$5, $6, $7 \n";
        $AFH{"$1$2$3"}->{mainLp} = "$5$6$7";
      }


     if ($_ =~ m/set (\w+)(\/)(\w+)[ \/0-9A-Za-z,]*workingLine(\s+)(\w+)(\/)(\d+)/  )
     {
#        print "found workingLine $1, $2 , $3 ,$5, $6, $7 \n";
        $AFH{"$1$2$3"}->{workingLine} = "$5$6$7";
      }

     if ($_ =~ m/set (\w+)(\/)(\w+)[,]?(\d*)[ \/0-9A-Za-z,]*logicalProcessor(\s+)(\w+)(\/)(\d+)/  )
     {
#        print "found LP $1, $2 , $3 , $4,$5, $6, $7 , $8\n";
        if ($4 eq  '') 
        {
        $AFH{"$1$2$3"}->{LP} = "$6$7$8";
       }
	else
       {
        $AFH{"$1$2$3,$4"}->{LP} = "$6$7$8";
       }
      }

     if ($_ =~ m/set Shelf Card(\/)(\w+)[ "\/0-9A-Za-z,]*cardType(\s+)(\w+)/  )
     {
        print "found Shelf Card  $2 \n";
        $SC{"Shelf Card $2"} = substr($4,0,6);
      }

   }
}


sub MapBtsBcn($$$)
{
    
   $prefix = shift;
   $outPrefix = shift;
   $bcnNum = shift;
         print "$prefix number is $bcnNum \n";
         if ( exists $AFN{"$prefix/$bcnNum"} )
         {
            print "$AFN{\"$prefix/$bcnNum\"} is the value to look for \n";
            $step2Val = $AFN{"$prefix/$bcnNum"};
            if ( exists $AFH{$step2Val}->{workingLine} )
            {
 		print "$AFH{$step2Val}->{workingLine} is the value in Step 2\n";
                $step3Val = $AFH{$step2Val}->{workingLine};

            }
            elsif ( exists $AFH{$step2Val}->{mainLp} )
            {

 		print "$AFH{$step2Val}->{mainLp} is the value in Step 2\n";
                $step3Val = $AFH{$step2Val}->{mainLp};

	    }
            else
            {
                print "No match was found for $step2Val in the AFH (workingLine or (mainLp)) hash. This is step2 in mapping BtsBcn to cardname\n";
            }
        
                if ( exists $AFH{$step3Val}->{mainCard} )
                {
 		   print "$AFH{$step3Val}->{mainCard} is the value in Step3 \n";
                  $shelfCard = $AFH{$step3Val}->{mainCard};
     	          if ( exists $SC{$shelfCard} )          
                 {
                    $slot = $shelfCard;
                    $slot=~s/Shelf Card//g;
                    print " $SC{$shelfCard} is the card and slot is $slot\n";
        		# Store into output hash
                    $outH{"$outPrefix/$bcnNum"}->{cardType}= $SC{$shelfCard};
                    $outH{"$outPrefix/$bcnNum"}->{slot} = $slot;
                 }
	          else
                  {
                    print "No match was found for $shelfCard in the SC hash. This is step4 in mapping BtsBcn to cardname\n";
                  }
	 	} # if for $AFH{$step3Val}->{mainCard}
                else
                {
                    print "No match was found for $step3Val in the AFH (mainCard) hash. This is step3 in mapping BtsBcn to cardname\n";
		}
		  
          } #  $AFN{"Bts/$bcnNum"}
          else
	  { 
                print "No match was found for $bcnNum in the AFN (applicationFramer ) hash. This is step1 in mapping BtsBcn to cardname\n";
          }        

} #sub

sub MapSctp_CpeBcn($$$)
{
    
   $prefix = shift;
   $outPrefix = shift;
   $sctpRestNum = shift;
         print "$prefix number is $sctpRestNum \n";
           $step1Val = $prefix."/".$sctpRestNum;

            if ( exists $AFH{$step1Val}->{LP} )
            {
 		print "$AFH{$step1Val}->{LP} is the value in Step 1\n";
                $step2Val = $AFH{$step1Val}->{LP};

            }
            else
            {
                print "No match was found for $step2Val in the AFH (LP)) hash. This is step1 in mapping Scep/CpeBcn to cardname\n";
            }
        
                if ( exists $AFH{$step2Val}->{mainLp} )
                {
 		   print "$AFH{$step2Val}->{mainLp} is the value in Step2 \n";
                  $step3Val= $AFH{$step2Val}->{mainLp};
                }
                if ( exists $AFH{$step3Val}->{mainCard} )
                {
 		   print "$AFH{$step3Val}->{mainCard} is the value in Step3 \n";
                  $shelfCard = $AFH{$step3Val}->{mainCard};
                }
     	          if ( exists $SC{$shelfCard} )          
                 {
                    $slot = $shelfCard;
                    $slot=~s/Shelf Card//g;
                    print " $SC{$shelfCard} is the card and slot is $slot\n";
					# Store into output hash
					if ($outPrefix eq "CpeBcnEp")
					{
						$outH{"$outPrefix/($sctpRestNum)"}->{cardType}= $SC{$shelfCard};
						$outH{"$outPrefix/($sctpRestNum)"}->{slot} = $slot;
					} 
					else
					{
						$outH{"$outPrefix/$sctpRestNum"}->{cardType}= $SC{$shelfCard};
						$outH{"$outPrefix/$sctpRestNum"}->{slot} = $slot;
					}
                 }
	          else
                  {
                    print "No match was found for $shelfCard in the SC hash. This is step4 in mapping BtsBcn to cardname\n";
                  }
} #sub

sub MapSbsBcn_ATM_Bssm($$$)
{
    
   $prefix = shift;
   $outPrefix = shift;
   $atmSbsBssmNum = shift;
   print "$prefix number is $atmSbsBssmNum \n";
   $step1Val = $prefix."/".$atmSbsBssmNum;

              #Common Step 1 for all three SbsBcn/AtmInterface/BssmIf

            if ( exists $AFH{$step1Val}->{interfaceName} )
            {
 		print "$AFH{$step1Val}->{interfaceName} is the value in Step 1\n";
                $step2Val = $AFH{$step1Val}->{interfaceName};

            }
            else
            {
                print "No match was found for $step2Val in the AFH (interfaceName) hash. This is step1 in mapping SbsBcn/AtmInterface/BssmIf to cardname\n";
            }
                #Step 2 for SbsBcn and AtmInterface 
        
                if ( $prefix eq "Sbs"  && exists $AFH{$step2Val}->{mainLp} )
                {
 		   print "$AFH{$step2Val}->{mainLp} is the value in Step2 \n";
                  $step3Val= $AFH{$step2Val}->{mainLp};
                }
                if ( $prefix eq "AtmIf" && exists $AFH{$step2Val}->{workingLine} )
                {
 		   print "$AFH{$step2Val}->{workingLine} is the value in Step2 \n";
                  $step3Val= $AFH{$step2Val}->{workingLine};
                }

                #Step 3 for SbsBcn and AtmInterface 
                if ( $prefix eq "Sbs" || $prefix eq "AtmIf" )
                {
 		   print "$AFH{$step3Val}->{mainCard} is the value in Step3 \n";
                  $shelfCard= $AFH{$step3Val}->{mainCard};
               
                }

                #Step 2 for BssmIf alone

                if ( $prefix eq "BssmIf"  && exists $AFH{$step2Val}->{mainCard} )
                {
 		   print "$AFH{$step2Val}->{mainCard} is the value in Step2 \n";
                  $shelfCard= $AFH{$step2Val}->{mainCard};
                }


		#Common final step for all 3. Mapping ShelfCard to actual cardname
     	          if ( exists $SC{$shelfCard} )          
                 {
                    $slot = $shelfCard;
                    $slot=~s/Shelf Card//g;
                    print " $SC{$shelfCard} is the card and slot is $slot\n";
        		# Store into output hash
			   
                    $outPrefix eq "SbsBcn" ? $outH{"$outPrefix/($atmSbsBssmNum)"}->{cardType}= $SC{$shelfCard} :  $outH{"$outPrefix/$atmSbsBssmNum"}->{cardType}= $SC{$shelfCard};
                    $outPrefix eq "SbsBcn" ? $outH{"$outPrefix/($atmSbsBssmNum)"}->{slot} = $slot : $outH{"$outPrefix/$atmSbsBssmNum"}->{slot} = $slot;
                 }
	          else
                  {
                    print "No match was found for $shelfCard in the SC hash. This is last Step in mapping SbsBcn/AtmInterface/BssmIf to cardname\n";
                  }
} #sub
sub MapLP($$)
{

   $LPNum = shift;
   $outPrefix = shift;
         print "Isshonumber is $LPNum \n";
                  $step1Val = "Lp/$LPNum";
                if ( exists $AFH{$step1Val}->{mainCard} )
                {
 		   print "$AFH{$step1Val}->{mainCard} is the value in Step1 \n";
                  $shelfCard = $AFH{$step1Val}->{mainCard};
     	          if ( exists $SC{$shelfCard} )          
                 {
                    $slot = $shelfCard;
                    $slot=~s/Shelf Card//g;
                    print " $SC{$shelfCard} is the card and slot is $slot\n";
        		# Store into output hash
                    $outH{"$outPrefix/$LPNum"}->{cardType}= $SC{$shelfCard};
                    $outH{"$outPrefix/$LPNum"}->{slot} = $slot;
                 }
	          else
                  {
                    print "No match was found for $shelfCard in the SC hash. This is step2 in mapping LP to cardname\n";
                  }
	 	} # if for $AFH{$step3Val}->{mainCard}
                else
                {
                    print "No match was found for $step1Val in the AFH (mainCard) hash. This is step1  in mapping LP (Logical Proc) to cardname\n";
		}
} #sub


sub gen_cbrs
{
   print "Dumper \n";
   print Dumper [ %AFN ];

  while (<BDF_FILE>)
  {
  chomp $_;
     ($pp, $rdfname, $cbrsname, $compName, $rest ) = split (" ", $_);
       $compName =~ s/\)//g;
       if  ($compName =~ m/BtsBcn\/(\d+)/ )
       {
	 MapBtsBcn("Bts", "BtsBcn", $1);
       }
       if  ($compName =~ m/CiuBcnIf\/(\d+)/ )
       {
	 MapBtsBcn("CiuIf",,"CiuBcnIf", $1);
       }
       if  ($compName =~ m/IsshoBcnIf\/(\d+)/ )
       {
	 MapBtsBcn("IsshoIf","IsshoBcnIf",$1);
       }
       if  ($compName =~ m/DiscoBcnIf\/(\d+)/ )
       {
	 MapBtsBcn("DiscoIf","DiscoBcnIf",$1);
       }
       if  ($compName =~ m/LogicalProcessor\/(\d+)/ )
       {
	 MapLP($1,"LogicalProcessor");
       }
       if  ($compName =~ m/Sctp\/(\d+)/ )
       {
	 MapSctp_CpeBcn("Sctp", "Sctp", $1);
       }
       if  ($compName =~ m/CpeBcnEp\/\((\d+),(\d+)/ )
       {
	 MapSctp_CpeBcn(BcnEp, "CpeBcnEp", "$1,$2");
       }
       if  ($compName =~ m/AtmInterface\/(\d+)/ )
       {
         MapSbsBcn_ATM_Bssm( "AtmIf", "AtmInterface", $1);
       }
       if  ($compName =~ m/SbsBcn\/\((\d+),(\d+)/ )
       {
         MapSbsBcn_ATM_Bssm("Sbs", "SbsBcn",  "$1,$2");
       }
       if  ($compName =~ m/BssmBcnIf\/(\d+)/ )
       {
         MapSbsBcn_ATM_Bssm("BssmIf", "BssmBcnIf", "$1");
       }
   }  

}


sub outputCBRS
{
   open(OUTFILE, ">$outFileName") || die("unable to open $outFileName \n");
    foreach $comp (keys %outH)
    {
        print OUTFILE "$outH{$comp}->{slot}   $outH{$comp}->{cardType} $comp \n"
    }
    `/usr/bin/sort -k  3,3 $outFileName > ${outFileName}.sort`;
     `echo "VERSION $rel\n" >  ${outFileName}`;
     `echo "#SlotNumber  CardName Component\n" >>  ${outFileName}`;
     `cat  ${outFileName}.sort >> $outFileName`;
     `rm  ${outFileName}.sort`
}

sub get_version
{
   $res=`cat $ascii_file|grep apcBase`;
    if ($res =~ m/apcBase_N(\w\w)\w+ / )
    {
         $rel = $1;
print "rel is $rel \n";
     }
}


sub remove_extra_files
{

  `rm ${temp_bdf_file}`;
}

trim_bdf;
open_ascii_view;
open_bdf_upd_file;
load_ascii_view;
get_version;
gen_cbrs;
outputCBRS;
remove_extra_files;
